#ifndef LIBELLE_EVE_OBJECTS
#define LIBELLE_EVE_OBJECTS

#include "FT800_Types.h"

typedef enum {taNone, taLeft, taCenter, taRight, taCenterX, taCenterY, taRightX} TTextAlign;

typedef struct Screen TScreen;

typedef unsigned long TPointer;

typedef struct tagObjInfo {
  void     *Obj;
  char     Type;
  char     Order;
  char     Flags;

  char     HitTag;
  int      HitX;
  int      HitY;
} TObjInfo;

typedef struct tagTouchStat {
  char Pressed;

  char Tag;
  int  X;
  int  Y;

  TObjInfo ActObjInfo;
} TTouchStat;

typedef void (*TDrawHandler)(TPointer aObj);

typedef void (*TEvtAction)();

typedef struct tagEvtSound {
  char SndAct;
  char Effect;
  char Pitch;
  char Volume;
} TEvtSound;

typedef const struct tagCEvent {
  TEvtAction Action;
  TEvtSound  Sound;
} TCEvent;

typedef struct tagEvent {
  TEvtAction Action;
  TEvtSound  Sound;
} TEvent;

typedef const struct tagCRect {
  int Left;
  int Top;
  int Width;
  int Height;
} TCRect;

typedef struct tagRect {
  int Left;
  int Top;
  int Width;
  int Height;
} TRect;

typedef struct tagLabel {
  TScreen         *OwnerScreen;
  char            Order;
  char            Visible;
  char            Opacity;
  char            Tag;
  int             Left;
  int             Top;
  int             Width;
  int             Height;
  char            *Caption;
  const code char *FontName;
  unsigned int    Font_Color;
  char            FontHandle;
  long            Source;
  char            Active;
  TEvent          *OnUp;
  TEvent          *OnDown;
  TEvent          *OnClick;
  TEvent          *OnPress;
} TLabel;

typedef struct tagCircle {
  TScreen      *OwnerScreen;
  char         Order;
  char         Visible;
  char         Opacity;
  char         Tag;
  int          Left;
  int          Top;
  int          Radius;
  char         Pen_Width;
  unsigned int Pen_Color;
  unsigned int Color;
  unsigned int Press_Color;
  char         Active;
  TEvent       *OnUp;
  TEvent       *OnDown;
  TEvent       *OnClick;
  TEvent       *OnPress;
} TCircle;

typedef struct tagEveSlider {
  TScreen      *OwnerScreen;
  char         Order;
  char         Visible;
  char         Opacity;
  char         Tag;
  int          Left;
  int          Top;
  int          Width;
  int          Height;
  unsigned int Background_Color;
  unsigned int Thumb_Color;
  unsigned int Color;
  unsigned int Press_Color;
  unsigned int Value;
  unsigned int Range;
  char         Flat;
  char         Active;
  TEvent       *OnUp;
  TEvent       *OnDown;
  TEvent       *OnClick;
  TEvent       *OnPress;
} TEveSlider;

typedef struct tagEveText {
  TScreen         *OwnerScreen;
  char            Order;
  char            Visible;
  char            Opacity;
  char            Tag;
  int             Left;
  int             Top;
  int             Width;
  int             Height;
  char            *Caption;
  TTextAlign      TextAlign;
  const code char *FontName;
  unsigned int    Font_Color;
  char            FontHandle;
  long            Source;
  char            Active;
  TEvent          *OnUp;
  TEvent          *OnDown;
  TEvent          *OnClick;
  TEvent          *OnPress;
} TEveText;

struct Screen {
  unsigned int   Color;
  unsigned int   Width;
  unsigned int   Height;
  unsigned short ObjectsCount;
  unsigned short LabelsCount;
  TLabel         *const code *Labels;
  unsigned short CirclesCount;
  TCircle        *const code *Circles;
  unsigned short EveSlidersCount;
  TEveSlider     *const code *EveSliders;
  unsigned short EveTextsCount;
  TEveText       *const code *EveTexts;
  unsigned long  DynResStart;
  unsigned short Active;
  unsigned short SniffObjectEvents;
  TEvent         *OnUp;
  TEvent         *OnDown;
  TEvent         *OnTagChange;
  TEvent         *OnPress;
};


// Object type constants
// Usage: VTFT stack internally
extern const VTFT_OT_LABEL;
extern const VTFT_OT_CIRCLE;
extern const VTFT_OT_EVESLIDER;
extern const VTFT_OT_EVETEXT;
// ~Object type constants

// Event type constants
// Usage: OnEvent
extern const VTFT_EVT_UP;
extern const VTFT_EVT_DOWN;
extern const VTFT_EVT_CLICK;
extern const VTFT_EVT_PRESS;
// ~Event type constants

// Sound action constants
// Usage: sound event action property and ProcessEvent
extern const VTFT_SNDACT_NONE;
extern const VTFT_SNDACT_PLAY;
extern const VTFT_SNDACT_STOP;
// ~Sound action constants

// Resource loading constants.
// Usage: DrawScreenO and LoadCurrentScreenResToGRAM
extern const VTFT_LOAD_RES_NONE;
extern const VTFT_LOAD_RES_STATIC;
extern const VTFT_LOAD_RES_DYNAMIC;
extern const VTFT_LOAD_RES_ALL;
// ~Resource loading constants

// Display effect constants
// Usage: DrawScreenO
extern const VTFT_DISPLAY_EFF_NONE;
extern const VTFT_DISPLAY_EFF_LIGHTS_FADE;
extern const VTFT_DISPLAY_EFF_LIGHTS_OFF;
// ~Display effect constants

// Stack flags
// Usage: internally used by VTFT stack
extern const VTFT_INT_REPAINT_ON_DOWN;
extern const VTFT_INT_REPAINT_ON_UP;
extern const VTFT_INT_BRING_TO_FRONT;
extern const VTFT_INT_INTRINSIC_CLICK_EFF;
// ~Stack flags

// Table of object draw handlers
// Use object type constants to access coresponding object draw handler
extern const TPointer DrawHandlerTable[44];
// ~Table of draw handler pointers

// Default configuration parameters
extern const TFT800PWM            VTFT_FT800_CONFIG_PWM;
extern const TFT800GPIO           VTFT_FT800_CONFIG_GPIO;
extern const TFT800Sound          VTFT_FT800_CONFIG_SOUND;
extern const TFT800Audio          VTFT_FT800_CONFIG_AUDIO;
extern const TFT800Display        VTFT_FT800_CONFIG_DISPLAY;
extern const TFT800Interrupt      VTFT_FT800_CONFIG_INTERRUPT;
extern const TFT800Touch          VTFT_FT800_CONFIG_TOUCH;
extern const TFT800TouchTransform VTFT_FT800_CONFIG_TOUCHTRANSFORM;

extern TTouchStat TouchS;

extern TScreen Screen2;

extern TEveSlider EveSlider_Basisachse;
extern TEvent     EveSlider_Basisachse_OnPress;
extern TEveSlider EveSlider_Achse2;
extern TEvent     EveSlider_Achse2_OnPress;
extern TCircle    Circle2;
extern TCircle    Circle4;
extern TCircle    Circle_Blase;
extern TEveText   EveText2;
extern TEveText   EveText3;
extern TLabel     Label10;
extern TLabel     Label3;
extern TLabel     Label4;
extern TLabel     Label1;
extern TLabel     Label5;
extern TLabel     Label6;
extern TLabel     Label2;
extern TLabel     Label7;
extern TLabel     Label8;
extern TLabel     Label9;
extern TLabel     Label11;
extern TLabel     Label12;
extern TLabel     Label13;
extern TLabel     Label14;

extern TLabel     *const code Screen2_Labels[14];
extern TCircle    *const code Screen2_Circles[3];
extern TEveSlider *const code Screen2_EveSliders[2];
extern TEveText   *const code Screen2_EveTexts[2];

extern TScreen *CurrentScreen;

/////////////////////////
// User Event Handlers
void EveSlider_Achse2OnPress();
void EveSlider_BasisachseOnPress();
/////////////////////////

/////////////////////////////////
// Captions
extern char EveSlider_Basisachse_Caption[];
extern char EveSlider_Achse2_Caption[];
extern char Circle2_Caption[];
extern char Circle4_Caption[];
extern char Circle_Blase_Caption[];
extern char EveText2_Caption[];
extern char EveText3_Caption[];
extern char Label10_Caption[];
extern char Label3_Caption[];
extern char Label4_Caption[];
extern char Label1_Caption[];
extern char Label5_Caption[];
extern char Label6_Caption[];
extern char Label2_Caption[];
extern char Label7_Caption[];
extern char Label8_Caption[];
extern char Label9_Caption[];
extern char Label11_Caption[];
extern char Label12_Caption[];
extern char Label13_Caption[];
extern char Label14_Caption[];

/////////////////////////////////
// Event Descriptors 
extern TEvent EveSlider_Basisachse_OnUpOnPress;
extern TEvent EveSlider_Achse2_OnUpOnPress;
/////////////////////////////////

void DrawScreenO(TScreen *aScreen, char aOptions);
void DrawScreen(TScreen *aScreen);
void DrawLabel(TLabel *ALabel);
void DrawCircle(TCircle *ACircle);
void DrawEveSlider(TEveSlider *AEveSlider);
void DrawEveText(TEveText *AEveText);
void ProcessVTFTStack();
void InitVTFTStack();

#endif
